function Inset_Corner(A,B,C,width){
	var splitLeft = B.clone().sub(A).normalize()
	var splitRight = C.clone().sub(B).normalize()
	
	var raw_attractor = splitLeft.clone().sub(splitRight);
	raw_attractor.normalize();
	
	var test_angle = new THREE.Vector3();
	test_angle.x = -splitLeft.z;
	test_angle.z = splitLeft.x;
	
	if(raw_attractor.length()==0){
		test_angle.multiplyScalar(width);
		return B.clone().sub(test_angle)
	}

	var back_angle = Math.acos(raw_attractor.dot(splitLeft));//angle between
	raw_attractor.divideScalar(Math.sin(back_angle));
	raw_attractor.multiplyScalar(width);
	
	if(test_angle.dot(splitRight)<0){//convex
		return B.clone().sub(raw_attractor)
	}else{ //concave
		return B.clone().add(raw_attractor)
	}
}

function Inset_Path(Target,width){
	
	var splits = []
	splits.push(Target[0].clone().sub(Target[Target.length-2]).normalize())
	for(var i=1;i<Target.length;i++){
		splits.push(Target[i].clone().sub(Target[i-1]).normalize())
	}
	
	//now to create the decension angles
	var raw_attractor
	var offset

	var geometry = []
	for(var i=0;i<Target.length-1;i++){
		//going toward the center is not the bisector
		raw_attractor = splits[i].clone().sub(splits[i+1]);
		raw_attractor.normalize();
		
		var back_angle = Math.acos(raw_attractor.dot(splits[i]));//angle between
		raw_attractor.divideScalar(Math.sin(back_angle));
		raw_attractor.multiplyScalar(width);
		var test_angle = new THREE.Vector3();
		test_angle.x = -splits[i].z;
		test_angle.z = splits[i].x;
		if(test_angle.dot(splits[i+1])<0){//convex
			geometry.push(Target[i].clone().sub(raw_attractor))
		}else{ //concave
			geometry.push(Target[i].clone().add(raw_attractor))
		}
	}
	geometry.push(geometry[0].clone());
	
	return geometry

}

function displayPath(target,height){
	var geometry = new THREE.Geometry()
	var v
	var t = 0
	for(var ii=0;ii<target.length;ii++){
		v = target[ii].clone()
		v.x += 10*Math.random() - 5
		v.y += 10*Math.random() - 5
		v.z += 10*Math.random() - 5
		//v.y = height + t*2
		t++
		geometry.vertices.push(v.clone())
	}
	v.add(new THREE.Vector3(0,5,0))
	geometry.vertices.push(v)
	var line = new THREE.Line(geometry)
	reality.add(line)
}

function orientation(target){
	var vectorA = target[1].clone().sub(target[0]).normalize()
	var vectorB = target[2].clone().sub(target[0]).normalize()
	var normal = vectorA.cross(vectorB)
	return (normal.y<0)
}

function findLineIntersection(A_S,A_E,B_S,B_E){
	// if the lines intersect, the result contains the x and y of the intersection (treating the lines as infinite) and booleans for whether line segment 1 or line segment 2 contain the point
	var denominator, a, b, numerator1, numerator2
	var result = new THREE.Vector3()
	denominator = ((B_E.z - B_S.z) * (A_E.x - A_S.x)) - ((B_E.x - B_S.x) * (A_E.z - A_S.z))
	if (denominator == 0) {
		return false
	}
	a = A_S.z - B_S.z;
	b = A_S.x - B_S.x;
	numerator1 = ((B_E.x - B_S.x) * a) - ((B_E.z - B_S.z) * b)
	numerator2 = ((A_E.x - A_S.x) * a) - ((A_E.z - A_S.z) * b)
	a = numerator1 / denominator
	b = numerator2 / denominator

	// if we cast these lines infinitely in both directions, they intersect here:
	result.x = A_S.x + (a * (A_E.x - A_S.x))
	result.z = A_S.z + (a * (A_E.z - A_S.z))
	/*
		// it is worth noting that this should be the same as:
		x = B_S.x + (b * (B_E.x - B_S.x));
		y = B_S.x + (b * (B_E.z - B_S.z));
		*/
	// if line1 is a segment and line2 is infinite, they intersect if:
	if (a <= 0 || a >= 1) {
		return false
	}
	// if line2 is a segment and line1 is infinite, they intersect if:
	if (b <= 0 || b >= 1) {
		return false
	}
	// if line1 and line2 are segments, they intersect if both of the above are true
	return result
}

function findPathIntersections(A,B){
	A.push(A[0])
	B.push(B[0])
	var product = []
	for(var ii=0;ii<A.length-1;ii++){
		for(var jj=0;jj<B.length-1;jj++){
			var result = findLineIntersection(A[ii],A[ii+1],B[jj],B[jj+1])
			if(result){
				product.push([ii,jj,result])
			}
		}
	}
	return product
}

function testFunc(){
	var A = this.border.flatten()
	var B = this.dad.floors[myindex+1].border.flatten()//root wall
	
	for(var ii=0;ii<A.length;ii++){
		A[ii].y = 0
	}
	
	for(var ii=0;ii<B.length;ii++){
		B[ii].y = 0
	}
	
	A.push(A[0])
	A = Inset_Path(A,-this.wallWidth)
	A.splice(A.length-1,1)
	
	B.push(B[0])
	B = Inset_Path(B,-this.dad.floors[myindex+1].wallWidth)
	B.splice(B.length-1,1)
	
	var vA = vector2path(A)
	var vB = vector2path(B)
	var solution_paths = new ClipperLib.Paths();
	
	var C = findPathIntersections(A,B)
	for(var ii=0;ii<C.length;ii++){
		var pos = new Point_Widget(C[ii][2],undefined)
		reality.add(pos.object)
	}
	var cpr = new ClipperLib.Clipper();
	cpr.AddPaths(vA, ClipperLib.PolyType.ptSubject, true);
	cpr.AddPaths(vB, ClipperLib.PolyType.ptClip, true);
	
	cpr.Execute(ClipperLib.ClipType.ctDifference, solution_paths, ClipperLib.PolyFillType.pftNonZero, ClipperLib.PolyFillType.pftNonZero);
	var product = []
	for(var ii=0;ii<solution_paths[0].length;ii++){
		var origin = new THREE.Vector3(solution_paths[0][ii].X,0,solution_paths[0][ii].Y)
		var real = new THREE.Vector3()
		var min_score = 99999999
		for(var jj=0;jj<A.length;jj++){
			if(A[jj].distanceTo(origin)<min_score){
				min_score = A[jj].distanceTo(origin)
				real = A[jj]
			}
		}
		for(var jj=0;jj<C.length;jj++){
			if(C[jj][2].distanceTo(origin)<min_score){
				min_score = C[jj][2].distanceTo(origin)
				real = C[jj][2]
			}
		}
		for(var jj=0;jj<B.length;jj++){
			if(B[jj].distanceTo(origin)<min_score){
				min_score = B[jj].distanceTo(origin)
				real = B[jj]
			}
		}
		product.push(real)
	}
	var tempBorder = new Border_Widget(this)
	for(var ii=0;ii<product.length;ii++){
		tempBorder.vertices.push(new Point_Widget(product[ii],tempBorder))
	}
	var surface = new Surface_Pro(tempBorder)
	reality.add(new THREE.Mesh(surface.genMesh()[0]))
	
}

function carve(clay_in,clay_out,knife){
	var I_in = findPathIntersections(clay_in,knife)
	var I_out = findPathIntersections(clay_out,knife)
	
	if(I_out.length==0){
		return [[],[]]
	}
	
	knife.reverse()
	var temp = decompose_Shape(knife)
	var polarity = false
	for(var ii=0;ii<temp.length;ii++){
		if(withinTri(clay_out[0],[knife[temp[ii][0]],knife[temp[ii][1]],knife[temp[ii][2]]])){
			polarity = true
			break
		}
	}
	
	var All_Cuts_Out = []
	var All_Cuts_In = []
	
	clay_out.push(clay_out[0])
	clay_in.push(clay_in[0])
	for(var ii=0;ii<clay_out.length-1;ii++){
		
		var Cut_Out = []
		var Cut_In = []
		
		for(var jj=0;jj<I_out.length;jj++){
			if(I_out[jj][0]==ii){
				for(var kk=0;kk<Cut_Out.length;kk++){
					if(clay_out[ii].distanceTo(Cut_Out[kk])>clay_out[ii].distanceTo(I_out[jj][2])){
						Cut_Out.splice(kk,0,I_out[jj][2])
						Cut_In.splice(kk,0,I_in[jj][2])
						break
					}
				}
				if(kk==Cut_Out.length){
					Cut_Out.push(I_out[jj][2])
					Cut_In.push(I_in[jj][2])
				}
			}
		}
		if(polarity){
			if(Cut_Out.length%2==1){
				polarity = !polarity
			}else{
				var n = clay_out[ii+1].clone().add(clay_out[ii+1].clone().sub(clay_out[ii]).normalize())
				Cut_Out.push(n)
				var n = clay_in[ii+1].clone().add(clay_in[ii+1].clone().sub(clay_in[ii]).normalize())
				Cut_In.push(n)
			}
			var n = clay_out[ii].clone().add(clay_out[ii].clone().sub(clay_out[ii+1]).normalize())
			Cut_Out.splice(0,0,n)
			var n = clay_in[ii].clone().add(clay_in[ii].clone().sub(clay_in[ii+1]).normalize())
			Cut_In.splice(0,0,n)
		}else{
			if(Cut_Out.length%2==1){
				polarity = !polarity
				var n = clay_out[ii+1].clone().add(clay_out[ii+1].clone().sub(clay_out[ii]).normalize())
				Cut_Out.push(n)
				var n = clay_in[ii+1].clone().add(clay_in[ii+1].clone().sub(clay_in[ii]).normalize())
				Cut_In.push(n)
			}
		}
		
		All_Cuts_Out.push(Cut_Out)
		All_Cuts_In.push(Cut_In)
	}
	
	clay_out.splice(clay_out.length-1,1)
	clay_in.splice(clay_in.length-1,1)
	
	return [All_Cuts_In,All_Cuts_Out]
}

function invert(target){
	var product = []
	for(var ii=target.length-1;ii>=0;ii--){
		product.push(target[ii])
	}
	return product
}

function clockwiseOrient(target){
	if(orientation(target)){
		return target
	}else{
		return invert(target)
	}
}

function vector2path(border){
	var product = [];
	for(var ii=0;ii<border.length;ii++){
		if(border[ii].x==0){border[ii].x=0}
		if(border[ii].z==0){border[ii].z=0}
		product.push({X: Math.round(border[ii].x),Y: Math.round(border[ii].z)})
	}
	return [product];
}

function findAngle(x,z){
	if(x==0){
		if(z>0){
			return Math.PI/2
		}else{
			return 3*Math.PI/2
		}
	}
	var product = Math.atan(z/x)
	if(x>0){product+=Math.PI}
	if(product<0){product+=2*Math.PI}
	return product
}

function Generate_Walls(pTarget,height,inset){
	var elevation = pTarget[0].y
	for(var ii=0;ii<pTarget.length;ii++){
		pTarget[ii].y = 0
	}
	pTarget.push(pTarget[0])
	var Target = Inset_Path(pTarget,inset)
	var product = [];
	for(var ii=0;ii<Target.length-1;ii++){
		var insets = [Target[ii],Target[ii+1]]
		var outsets = [pTarget[ii],pTarget[ii+1]]
		product.push(new PRIM_WALL(insets,outsets,elevation,elevation+height))
		
		/*var border = new Border_Widget()
		border.vertices.push(
			new Point_Widget( Target[ii].clone(),border),
			new Point_Widget( Target[ii].clone().add(new THREE.Vector3(0,height)),border),
			new Point_Widget( Target[ii+1].clone().add(new THREE.Vector3(0,height)),border),
			new Point_Widget( Target[ii+1].clone(),border)
		)
		var surface = new Surface_Pro(border)
		product.push(surface)*/
	}
	return product
}